//
// $History: CheckSum.pas $
// 
// *****************  Version 7  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 6  *****************
// User: Lvk          Date: 12.09.98   Time: 17:36
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 5  *****************
// User: Lvk          Date: 9.09.98    Time: 19:15
// Updated in $/Delphi 4.0/StreamFilter
// - Made library Delphi 3.x compatible
// - Fixed bug in encryption base classes
// - Replaced old email address
// 
// *****************  Version 4  *****************
// User: Lvk          Date: 22.07.98   Time: 17:19
// Updated in $/Delphi 4.0/StreamFilter
// Fixed some comment blocks
// 
// *****************  Version 3  *****************
// User: Lvk          Date: 21.07.98   Time: 23:50
// Updated in $/Delphi 4.0/StreamFilter
// Removed comment block documenting use
// 
// *****************  Version 2  *****************
// User: Lvk          Date: 20.07.98   Time: 20:50
// Updated in $/Delphi 4.0/StreamFilter
// Added documentation for first release
// 
// *****************  Version 1  *****************
// User: Lvk          Date: 19.07.98   Time: 23:50
// Created in $/Delphi 4.0/StreamFilter
// Base CheckSum class
// 8, 16 and 32-bit byte-sum CheckSum classes
//

unit CheckSum;

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net

interface

uses
	SysUtils, AuxFuncs;

type
{ TBasicCheckSum class }
	TBasicCheckSum = class
	private
		FCheckSumSize	: Integer;
		FName					: string;
		function GetString: string;
		function GetTheByte(Index: Integer): Byte; virtual;
	protected
		function GetByte(Index: Integer): Byte; virtual; abstract;
	public
		constructor Create(AName: string; ACheckSumSize: Integer);
		destructor Destroy; override;

		procedure Reset; virtual; abstract;
		procedure Process(const Data; Size: Integer); virtual; abstract;

		property Name: string read FName;
		property Size: Integer read FCheckSumSize;
		property Bytes[Index: Integer]: Byte read GetTheByte;
		property AsString: string read GetString;
	end; // class TBasicCheckSum

{ TBasicCheckSum8Bit class }
	TBasicCheckSum8Bit = class(TBasicCheckSum)
	private
		FSum	: Byte;
	protected
		function GetByte(Index: Integer): Byte; override;
		procedure SetCheckSum(Value: Byte); virtual;
	public
		constructor Create(AName: string);
		procedure Reset; override;

		property Sum: Byte read FSum;
	end; // class TBasicCheckSum8Bit

{ TSum8 class }
	TSum8	= class(TBasicCheckSum8Bit)
	public
		constructor Create;
		procedure Process(const Data; Size: Integer); override;
	end; // class TSum8

{ TBasicCheckSum16Bit class }
	TBasicCheckSum16Bit = class(TBasicCheckSum)
	private
		FSum	: Word;
	protected
		function GetByte(Index: Integer): Byte; override;
		procedure SetCheckSum(Value: Word); virtual;
	public
		constructor Create(AName: string);
		procedure Reset; override;

		property Sum: Word read FSum;
	end; // class TBasicCheckSum16Bit

{ TSum16 class }
	TSum16	= class(TBasicCheckSum16Bit)
	public
		constructor Create;
		procedure Process(const Data; Size: Integer); override;
	end; // class TSum16

{ TBasicCheckSum32Bit class }
	TBasicCheckSum32Bit = class(TBasicCheckSum)
	private
		FSum	: LongWord;
	protected
		function GetByte(Index: Integer): Byte; override;
		procedure SetCheckSum(Value: LongWord); virtual;
	public
		constructor Create(AName: string);
		procedure Reset; override;

		property Sum: LongWord read FSum;
	end; // class TBasicCheckSum32Bit

{ TSum32 class }
	TSum32	= class(TBasicCheckSum32Bit)
	public
		constructor Create;
		procedure Process(const Data; Size: Integer); override;
	end; // class TSum32

implementation

resourcestring
	SGetByteRange = 'Call to %s.GetByte [property Bytes] with index <> [0..%d]';
	S8BitSum			= '8-bit byte-sum';
	S16BitSum			= '16-bit byte-sum';
	S32BitSum			= '32-bit byte-sum';

{ TBasicCheckSum }

constructor TBasicCheckSum.Create(AName: string; ACheckSumSize: Integer);
begin
	// Set private members for properties
	FCheckSumSize := ACheckSumSize;
	FName := AName;

	// Reset CheckSum
	Reset;
end; // constructor TBasicCheckSum.Create

destructor TBasicCheckSum.Destroy;
begin
	// Nothing to destroy
end; // destructor TBasicCheckSum.Destroy

function TBasicCheckSum.GetString: string;
const
	HexDigits	: array[0..15] of Char = '0123456789abcdef';
var
	i					: Integer;
	B					: Byte;
begin
	Result := '';

	// Concatenate hex-digits from CheckSum bytes
	for i := 1 to Size do
	begin
		B := Bytes[i-1];
		Result := Result + HexDigits[B shr 4] + HexDigits[B and 15];
	end; // for i := 1 to Size
end; // function TBasicCheckSum.GetString

function TBasicCheckSum.GetTheByte(Index: Integer): Byte;
begin
	if (Index < 0) or (Index >= FCheckSumSize) then
		raise ERangeError.CreateFmt(SGetByteRange, [ClassName, 0, Size-1])
	else
		Result := GetByte(Index);
end; // function TBasicCheckSum.GetTheByte

{ TBasicCheckSum8Bit }

constructor TBasicCheckSum8Bit.Create(AName: string);
begin
	inherited Create(AName, 1);
end; // constructor TBasicCheckSum8Bit.Create

function TBasicCheckSum8Bit.GetByte(Index: Integer): Byte;
begin
	Result := FSum;
end; // function TBasicCheckSum8Bit.GetByte

procedure TBasicCheckSum8Bit.Reset;
begin
	SetCheckSum(0);
end; // procedure TBasicCheckSum8Bit.Reset

procedure TBasicCheckSum8Bit.SetCheckSum(Value: Byte);
begin
	FSum := Value;
end; // procedure TBasicCheckSum8Bit.SetCheckSum

{ TSum8 }

constructor TSum8.Create;
begin
	inherited Create(S8BitSum);
end; // constructor TSum8.Create

{$R-,Q-}
procedure TSum8.Process(const Data; Size: Integer);
var
	P	: ^Byte;
	I	: Integer;
	S	: Byte;
begin
	S := Sum;
	P := @Data;
	for I := 1 to Size do
	begin
		Inc(S, P^);
		Inc(P);
	end; // for I := 1 to Size
	SetCheckSum(S);
end; // procedure TSum8.Process
{$R+,Q+}

{ TBasicCheckSum16Bit }

constructor TBasicCheckSum16Bit.Create(AName: string);
begin
	inherited Create(AName, 2);
end; // constructor TBasicCheckSum16Bit.Create

function TBasicCheckSum16Bit.GetByte(Index: Integer): Byte;
begin
	case Index of
		0	: Result := (FSum shr 8);
		1	: Result := FSum and 255;
	else
		Result := 0;
	end; // case Index
end; // function TBasicCheckSum16Bit.GetByte

procedure TBasicCheckSum16Bit.Reset;
begin
	SetCheckSum(0);
end; // procedure TBasicCheckSum16Bit.Reset

procedure TBasicCheckSum16Bit.SetCheckSum(Value: Word);
begin
	FSum := Value;
end; // procedure TBasicCheckSum16Bit.SetCheckSum

{ TSum16 }

constructor TSum16.Create;
begin
	inherited Create(S16BitSum);
end; // constructor TSum16.Create

{$R-,Q-}
procedure TSum16.Process(const Data; Size: Integer);
var
	P	: ^Byte;
	I	: Integer;
begin
	P := @Data;
	for I := 1 to Size do
	begin
		Inc(FSum, P^);
		Inc(P);
	end; // for I := 1 to Size
end; // procedure TSum16.Process
{$R+,Q+}

{ TBasicCheckSum32Bit }

constructor TBasicCheckSum32Bit.Create(AName: string);
begin
	inherited Create(AName, 4);
end; // constructor TBasicCheckSum32Bit.Create

function TBasicCheckSum32Bit.GetByte(Index: Integer): Byte;
begin
	case Index of
		0	: Result := (FSum shr 24);
		1	: Result := (FSum shr 16) and 255;
		2	: Result := (FSum shr 8) and 255;
		3	: Result := (FSum and 255);
	else
		Result := 0;
	end; // case Index
end; // function TBasicCheckSum32Bit.GetByte

procedure TBasicCheckSum32Bit.Reset;
begin
	SetCheckSum(0);
end; // procedure TBasicCheckSum32Bit.Reset

procedure TBasicCheckSum32Bit.SetCheckSum(Value: LongWord);
begin
	FSum := Value;
end; // procedure TBasicCheckSum32Bit.SetCheckSum

{ TSum32 }

constructor TSum32.Create;
begin
	inherited Create(S32BitSum);
end; // constructor TSum32.Create

{$R-,Q-}
procedure TSum32.Process(const Data; Size: Integer);
var
	P	: ^Byte;
	I	: Integer;
begin
	P := @Data;
	for I := 1 to Size do
	begin
		Inc(FSum, P^);
		Inc(P);
	end; // for I := 1 to Size
end; // procedure TSum32.Process
{$R+,Q+}

end. // unit CheckSum
