//
// $History: CheckSumStream.pas $
// 
// *****************  Version 5  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 4  *****************
// User: Lvk          Date: 12.09.98   Time: 17:36
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 3  *****************
// User: Lvk          Date: 9.09.98    Time: 19:15
// Updated in $/Delphi 4.0/StreamFilter
// - Made library Delphi 3.x compatible
// - Fixed bug in encryption base classes
// - Replaced old email address
// 
// *****************  Version 2  *****************
// User: Lvk          Date: 20.07.98   Time: 20:50
// Updated in $/Delphi 4.0/StreamFilter
// Added documentation for first release
// 
// *****************  Version 1  *****************
// User: Lvk          Date: 19.07.98   Time: 23:50
// Created in $/Delphi 4.0/StreamFilter
// Stream that computes checksum when data is filtered through it
//

unit CheckSumStream;

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net

interface

// This class takes two other classes as parameters;
//   ACoreStream: TStream
//   ACheckSumClass: TBasicCheckSum
//
// As you read and write from/to the class (to the core stream, through this
// class), the checksum is calculated. To get the checksum, either use the
// properties of the CheckSumClass class directly, or use the CheckSum
// property to get the checksum as a hex string. Please remember that the
// length of the checksum string depends on the size of the checksum returned
// by the checksum class.
//
// As usual, the two DoFree??? parameters to the stream decides if the
// streams .Free method should also free the given objects

uses
	CheckSum, Classes, StreamFilters;

type
	TCheckSumStream = class(TBasicStreamFilter)
	private
		FCheckSumClass			: TBasicCheckSum;
		FFreeCheckSumClass	: Boolean;
    function GetCheckSum: string;
	public
{$IFDEF VER120}
// Delphi 4.x specific code
		constructor Create(ACoreStream: TStream; ACheckSumClass: TBasicCheckSum;
			DoFreeCore: Boolean=FALSE; DoFreeCheckSumClass: Boolean=FALSE);
{$ELSE}
// Delphi 3.x specific code
		constructor Create(ACoreStream: TStream; ACheckSumClass: TBasicCheckSum;
			DoFreeCore, DoFreeCheckSumClass: Boolean);
{$ENDIF}
		destructor Destroy; override;

		function Read(var Buffer; Count: LongInt): LongInt; override;
		function Write(const Buffer; Count: Longint): Longint; override;
		function Seek(Offset: Longint; Origin: Word): Longint; override;

		property CheckSumClass: TBasicCheckSum read FCheckSumClass;
		property FreeCheckSumClass: Boolean read FFreeCheckSumClass;

		property CheckSum: string read GetCheckSum;
	end;

implementation

{ TCheckSumStream }

constructor TCheckSumStream.Create(ACoreStream: TStream; ACheckSumClass: TBasicCheckSum;
	DoFreeCore: Boolean; DoFreeCheckSumClass: Boolean);
begin
	inherited Create(ACoreStream, DoFreeCore);
	FCheckSumClass := ACheckSumClass;
	FFreeCheckSumClass := DoFreeCheckSumClass;
end; // constructor TCheckSumStream.Create

destructor TCheckSumStream.Destroy;
begin
	if FreeCheckSumClass then
		CheckSumClass.Free;
	inherited;
end; // destructor TCheckSumStream.Destroy

function TCheckSumStream.GetCheckSum: string;
begin
	Result := CheckSumClass.AsString;
end; // function TCheckSumStream.GetCheckSum

function TCheckSumStream.Read(var Buffer; Count: Integer): LongInt;
begin
	Result := CoreStream.Read(Buffer, Count);
	CheckSumClass.Process(Buffer, Result);
end; // function TCheckSumStream.Read

function TCheckSumStream.Seek(Offset: Integer; Origin: Word): Longint;
begin
	if (Offset = 0) and (Origin = soFromBeginning) then
		CheckSumClass.Reset;
	Result := CoreStream.Seek(Offset, Origin);
end;

function TCheckSumStream.Write(const Buffer; Count: Integer): Longint;
begin
	Result := CoreStream.Write(Buffer, Count);
	CheckSumClass.Process(Buffer, Result);
end;

end.
 