unit ComplianceForm;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, CheckSum, CheckSumStream, CRC, MD5, SHA, Cryptography, IDEA, Blowfish, RC5, RC6,
	zLibStream, CryptographyStream;

type
	TfrmMainForm = class(TForm)
		memoLog: TMemo;
    btnStartTest: TButton;
		procedure btnStartTestClick(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		Data			: PChar;
		DataSize	: Integer;

		procedure InitData;
		procedure FreeData;
		procedure Log(S: string);

		procedure TestCheckSum(CheckSum: TBasicCheckSum; ResultSum: string);

		procedure TestCompression(ResultSum: string);
		procedure TestEncryption(Encryption: TCryptography; ResultSum: string);

		procedure Compare(Name: string; ResultSum,ExpectedSum: string);
	end;

var
	frmMainForm: TfrmMainForm;

implementation

{$R *.DFM}

procedure TfrmMainForm.btnStartTestClick(Sender: TObject);
begin
	memoLog.Clear;
	InitData;

	TestCheckSum(TSum8.Create,	'00');
	TestCheckSum(TSum16.Create,	'2600');
	TestCheckSum(TSum32.Create,	'00042600');
	TestCheckSum(TCRC16.Create,	'18c6');
	TestCheckSum(TCRC32.Create,	'b2a15c02');
	TestCheckSum(TSHA.Create,		'14a5e58cb4c45e9edc19baa40f05edbb24952d62');
	TestCheckSum(TMD5.Create,		'cafd9b22b291f58872c779821326bacd');

	TestCompression('da39a3ee5e6b4b0d3255bfef95601890afd80709');
{$IFDEF VER120}
// Delphi 4.x specific code
	TestEncryption(TIDEA.Create('password to use that is long'), '68b2841e0350cdf0525077cf58a56fc5217e4ec3');
	TestEncryption(TBlowfish.Create('password to use that is long'), '9c5e88bfc4b6d8edfc530f1817a07dd0ee3cc95d');
	TestEncryption(TRC5.Create('password to use that is long'), 'f410f89fa37a8aeeebaa804ae801f66e8392c05a');
	TestEncryption(TRC6.Create('password to use that is long'), 'a8124ee45f58c6923863a4cbb12b7cf114c926e3');
{$ELSE}
// Delphi 3.x specific code
	TestEncryption(TIDEA.CreateKey('password to use that is long'), '68b2841e0350cdf0525077cf58a56fc5217e4ec3');
	TestEncryption(TBlowfish.CreateKey('password to use that is long'), '9c5e88bfc4b6d8edfc530f1817a07dd0ee3cc95d');
	TestEncryption(TRC5.CreateKey('password to use that is long'), 'f410f89fa37a8aeeebaa804ae801f66e8392c05a');
	TestEncryption(TRC6.CreateKey('password to use that is long'), 'a8124ee45f58c6923863a4cbb12b7cf114c926e3');
{$ENDIF}
	FreeData;
end;

procedure TfrmMainForm.Compare(Name, ResultSum, ExpectedSum: string);
begin
	while Length(Name) < 60 do
		Name := Name + ' ';

	if (ResultSum = ExpectedSum) then
		Log(Name +'  PASSED')
	else
		Log(Name + '* FAILED *');
end;

procedure TfrmMainForm.FreeData;
begin
	FreeMem(Data, DataSize);
	Data := nil;
	DataSize := 0;
end;

procedure TfrmMainForm.InitData;
var
	i	: Integer;
begin
	DataSize := 2048;
	GetMem(Data, DataSize);

	for i := 0 to DataSize-1 do
		Data[i] := Chr(i xor (i shl (i and 7) xor 170));
end;

procedure TfrmMainForm.Log(S: string);
begin
	memoLog.Lines.Add(S);
end;

procedure TfrmMainForm.TestCheckSum(CheckSum: TBasicCheckSum; ResultSum: string);
var
	Stream	: TStream;
begin
	Stream := TCheckSumStream.Create(TMemoryStream.Create, CheckSum, True, True);

	Stream.Write(Data^, DataSize);
	Compare(CheckSum.Name, CheckSum.AsString, ResultSum);

	Stream.Free;
end;

procedure TfrmMainForm.TestCompression(ResultSum: string);
var
	MemoryStream					: TStream;
	CompressionStream			: TStream;
	CheckSumAfter					: TBasicCheckSum;
	CheckSumAfterStream		: TStream;
	CheckSumBefore				: TBasicCheckSum;
	CheckSumBeforeStream	: TStream;
	Temp									: PChar;
begin
	// Allocate classes for compression
	MemoryStream := TMemoryStream.Create;
	CheckSumAfter := TSHA.Create;
	CheckSumAfterStream := TCheckSumStream.Create(MemoryStream, CheckSumAfter, False, False);
	CompressionStream := TzLibCompressionStream.Create(CheckSumAfterStream, False, clBestCompression,
		32768);
	CheckSumBefore := TSHA.Create;
	CheckSumBeforeStream := TCheckSumStream.Create(CompressionStream, CheckSumBefore, False, False);

	// Now do the compression and validate the result
	CheckSumBeforeStream.Write(Data^, DataSize);
	Compare('zLib Compression', CheckSumAfter.AsString, ResultSum);

	// Free up classes not needed for decompression
	CheckSumBeforeStream.Free;
	CompressionStream.Free;
	CheckSumAfterStream.Free;
	CheckSumAfter.Free;

	// Allocate classes needed for decompression
	GetMem(Temp, DataSize);
	CheckSumAfter := TSHA.Create;
	MemoryStream.Seek(0, soFromBeginning);
	CompressionStream := TzLibDecompressionStream.Create(MemoryStream, False, 32768);
	CheckSumAfterStream := TCheckSumStream.Create(CompressionStream, CheckSumAfter, False, False);

	// Now do the decompression and validate the result
	CheckSumAfterStream.Read(Temp^, DataSize);
	Compare('zLib Decompression', CheckSumAfter.AsString, CheckSumBefore.AsString);

	// Free up rest of classes
	CheckSumAfterStream.Free;
	CheckSumAfter.Free;
	CheckSumBefore.Free;
	CompressionStream.Free;
	MemoryStream.Free;

	FreeMem(Temp, DataSize);
end;

procedure TfrmMainForm.TestEncryption(Encryption: TCryptography;
  ResultSum: string);
var
	Stream		: TStream;
	CheckSum	: TBasicCheckSum;
begin
	CheckSum := TSHA.Create;
	Stream := TEncryptionStream.Create(TCheckSumStream.Create(TMemoryStream.Create, CheckSum, True, False),
		Encryption, True, True);

	Stream.Write(Data^, DataSize);
	Compare(Encryption.Name, CheckSum.AsString, ResultSum);

	Stream.Free;
	CheckSum.Free;
end;

end.

