//
// $History: IDEA.pas $
// 
// *****************  Version 10  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 9  *****************
// User: Lvk          Date: 12.09.98   Time: 17:36
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 8  *****************
// User: Lvk          Date: 9.09.98    Time: 19:15
// Updated in $/Delphi 4.0/StreamFilter
// - Made library Delphi 3.x compatible
// - Fixed bug in encryption base classes
// - Replaced old email address
// 
// *****************  Version 7  *****************
// User: Lvk          Date: 22.07.98   Time: 17:20
// Updated in $/Delphi 4.0/StreamFilter
// Fixed some comment blocks
// 
// *****************  Version 6  *****************
// User: Lvk          Date: 21.07.98   Time: 23:51
// Updated in $/Delphi 4.0/StreamFilter
// Made the base class (TCryptography) expand the key and make a binary
// key of the correct size, thereby reducing code in each descendant
// class.
// 
// *****************  Version 5  *****************
// User: Lvk          Date: 20.07.98   Time: 20:50
// Updated in $/Delphi 4.0/StreamFilter
// Added documentation for first release
// 
// *****************  Version 4  *****************
// User: Lvk          Date: 18.07.98   Time: 0:13
// Updated in $/Delphi 4.0/StreamFilter
// Fixed copyright message, and added reference to original paper
// 
// *****************  Version 3  *****************
// User: Lvk          Date: 17.07.98   Time: 23:51
// Updated in $/Delphi 4.0/StreamFilter
// Added copyright message
// 
// *****************  Version 2  *****************
// User: Lvk          Date: 17.07.98   Time: 20:44
// Updated in $/Delphi 4.0/Cryptography
// Nothing done
// 
// *****************  Version 1  *****************
// User: Lvk          Date: 17.07.98   Time: 20:43
// Created in $/Delphi 4.0/Cryptography
// IDEA encryption class
//

unit IDEA;

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net
//
// The IDEA algorithm is a patented encryption algorithm, made free for
// non-profit use. Any use of this algorithm in a commercial product
// carries patent rights to be abided.
//
// The original paper that were used for implementing this algorithm
// were found at ftp://ftp.funet.fi/pub/crypt/papers/

interface

uses
	SysUtils, Cryptography, AuxFuncs;

const
	Rounds		= 8;
	KeyLength	= (Rounds * 6) + 4;

type
	TIDEAKey		= array[0..KeyLength-1] of Word;
	TIDEABlock	= array[1..4] of Word;

{ TIDEA class }
	TIDEA	= class(TCryptography)
	private
		Z	: TIDEAKey;
		K	: TIDEAKey;
		procedure ExpandKeys; virtual;
		procedure InvertKeys; virtual;
	protected
		procedure CalculateSubKeys; override;
		procedure Cipher(var Block: TIDEABlock; const Keys: TIDEAKey); virtual;
	public
{$IFDEF VER120}
// Delphi 4.x specific code
		constructor Create; overload;
		constructor Create(AKey: string); overload;
{$ELSE}
// Delphi 3.x specific code
		constructor Create;
		constructor CreateKey(AKey: string); 
{$ENDIF}
		destructor Destroy; override;

		function EncipherBlock(var Block): Boolean; override;
		function DecipherBlock(var Block): Boolean; override;
	end; // class TIDEA

implementation

const
	Maxim	= 65537;
	
{ TIDEA }

procedure TIDEA.CalculateSubKeys;
begin
	ExpandKeys;
	InvertKeys;
end; // procedure TIDEA.CalculateSubKeys

constructor TIDEA.Create;
begin
	inherited Create('IDEA', 8, 8 * 4);
end; // constructor TIDEA.Create

{$R-,Q-}
procedure TIDEA.Cipher(var Block: TIDEABlock; const Keys: TIDEAKey);
var
	x1,x2,x3,x4	: Word;
	t1,t2				: Word;
	pz					: ^Word;
	r						: Integer;

	function Mul(a,b: Word): Word;
	var
		p	: LongWord;
	begin
		if (a > 0) then
		begin
			if (b > 0) then
			begin
				p := LongWord(a)*b;
				b := p and $ffff;
				a := p shr 16;
				Result := ((b - a) + Ord(b < a));
			end // if (b > 0)
			else
				Result := 1 - a;
		end // if (a > 0)
		else
			Result := 1 - b;
	end; // function TIDEA.Cipher.Mul

begin
//       x1 = *in++;  x2 = *in++;
	x1 := Block[1];
	x2 := Block[2];
//       x3 = *in++;  x4 = *in;
	x3 := Block[3];
	x4 := Block[4];

	pz := @Keys;
	for r := 1 to Rounds do
	begin
//             MUL(x1,*Z++);
		x1 := Mul(x1, pz^);
		Inc(pz);

//             x2 += *Z++;
		x2 := x2 + pz^;
		Inc(pz);

//             x3 += *Z++;
		x3 := x3 + pz^;
		Inc(pz);

//             MUL(x4, *Z++);
		x4 := Mul(x4, pz^);
		Inc(pz);

//             t2 = x1^x3;
		t2 := x1 xor x3;

//             MUL(t2, *Z++);
		t2 := Mul(t2, pz^);
		Inc(pz);

//             t1 = t2 + (x2^x4);
		t1 := t2 + (x2 xor x4);

//             MUL(t1, *Z++);
		t1 := Mul(t1, pz^);
		Inc(pz);

//             t2 = t1+t2;
		t2 := (t1 + t2);

//             x1 ^= t1;
		x1 := x1 xor t1;

//             x4 ^= t2;
		x4 := x4 xor t2;

//             t2 ^= x2;
		t2 := t2 xor x2;

//             x2 = x3^t1;
		x2 := x3 xor t1;

//             x3 = t2;
		x3 := t2;
	end; // for r := 1 to Rounds

//       MUL(x1, *Z++);
	x1 := Mul(x1, pz^);
	Inc(pz);

//       *out++ = x1;
	Block[1] := x1;

//       *out++ = x3 + *Z++;
	Block[2] := x3 + pz^;
	Inc(pz);

//       *out++ = x2 + *Z++;
	Block[3] := x2 + pz^;
	Inc(pz);

//       MUL(x4, *Z);
	x4 := Mul(x4, pz^);

//       *out = x4;
	Block[4] := x4;
end; // procedure TIDEA.Cipher
{$R+,Q+}

{$IFDEF VER120}
// Delphi 4.x specific code
constructor TIDEA.Create(AKey: string);
begin
	inherited Create('IDEA', 8, 8 * 4, AKey);
end; // constructor TIDEA.Create
{$ELSE}
// Delphi 3.x specific code
constructor TIDEA.CreateKey(AKey: string);
begin
	inherited CreateKey('IDEA', 8, 8 * 4, AKey);
end; // constructor TIDEA.Create
{$ENDIF}

function TIDEA.DecipherBlock(var Block): Boolean;
begin
	Cipher(TIDEABlock(Block), K);
	Result := TRUE;
end; // function TIDEA.DecipherBlock

destructor TIDEA.Destroy;
begin
	// Local destroy here
	inherited Destroy;
end; // destructor TIDEA.Destroy

function TIDEA.EncipherBlock(var Block): Boolean;
begin
	Cipher(TIDEABlock(Block), Z);
	Result := TRUE;
end; // function TIDEA.EncipherBlock

{$R-,Q-}
procedure TIDEA.ExpandKeys;
var
	i			: Integer;
begin
	// Copy the key into Z
	Move(KeyPtr^, Z, KeySize);

	// Now generate the encryption subkeys
	for i := 8 to KeyLength-1 do
	begin
		if (((i+2) mod 8) = 0) then
			Z[i] := (Z[i- 7] shl 9) xor (Z[i-14] shr 7)
		else if (((i+1) mod 8) = 0) then
			Z[i] := (Z[i-15] shl 9) xor (Z[i-14] shr 7)
		else
			Z[i] := (Z[i- 7] shl 9) xor (Z[i- 6] shr 7);
	end; // for j := 8 to KeyLength-1
end; // procedure TIDEA.ExpandKeys

procedure TIDEA.InvertKeys;
type
	PWord		= ^Word;
var
	j					: Integer;
	pz, pp		: PWord;
	t1,t2,t3	: Word;

	function Inv(I: Integer): Integer;
	var
		n1,n2,q,r,b1,b2,t	: Integer;
	begin
		if (I = 0) then
			Result := 0
		else begin
			n1 := Maxim;
			n2 := I;
			b2 := 1;
			b1 := 0;
			repeat
				r := (n1 mod n2);
				q := (n1-r) div n2;
				if (r = 0) then
				begin
					if (b2 < 0) then
						b2 := Maxim+ b2;
				end // if (r = 0)
				else begin
					n1 := n2;
					n2 := r;
					t := b2;
					b2 := b1 - q * b2;
					b1 := t;
				end; // if (r = 0) ... else
			until (r = 0); // repeat
			Result := b2;
		end; // if (I = 0) ... else
		Result := (Result and $ffff);
	end; // function TIDEA.InvertKeys.Inv

begin
	pz := @Z;
	pp := @K;	Inc(pp, KeyLength);

//       t1 = inv(*Z++);
	t1 := Inv(pz^);
	Inc(pz);

//       t2 = -*Z++;
	t2 := -pz^;
	Inc(pz);

//       t3 = -*Z++;
	t3 := -pz^;
	Inc(pz);

//       *--p = inv(*Z++);
	Dec(pp);
	pp^ := Inv(pz^);
	Inc(pz);

//       *--p = t3;
	Dec(pp);
	pp^ := t3;

//       *--p = t2;
	Dec(pp);
	pp^ := t2;

//       *--p = t1;
	Dec(pp);
	pp^ := t1;

	for j := 1 to Rounds-1 do
	begin
//             t1 = *Z++;
		t1 := pz^;
		Inc(pz);

//             *--p = *Z++;
		Dec(pp);
		pp^ := pz^;
		Inc(pz);

//             *--p = t1;
		Dec(pp);
		pp^ := t1;

//             t1 = inv(*Z++);
		t1 := Inv(pz^);
		Inc(pz);

//             t2 = -*Z++;
		t2 := -pz^;
		Inc(pz);

//             t3 = -*Z++;
		t3 := -pz^;
		Inc(pz);

//             *--p = inv(*Z++);
		Dec(pp);
		pp^ := Inv(pz^);
		Inc(pz);

//             *--p = t2;
		Dec(pp);
		pp^ := t2;

//             *--p = t3;
		Dec(pp);
		pp^ := t3;

//             *--p = t1;
		Dec(pp);
		pp^ := t1;
	end; // for j := 1 to Rounds-1

//       t1 = *Z++;
	t1 := pz^;
	Inc(pz);

//       *--p = *Z++;
	Dec(pp);
	pp^ := pz^;
	Inc(pz);

//       *--p = t1;
	Dec(pp);
	pp^ := t1;

//       t1 = inv(*Z++);
	t1 := Inv(pz^);
	Inc(pz);

//       t2 = -*Z++;
	t2 := -pz^;
	Inc(pz);

//       t3 = -*Z++;
	t3 := -pz^;
	Inc(pz);

//       *--p = inv(*Z++);
	Dec(pp);
	pp^ := Inv(pz^);

//       *--p = t3;
	Dec(pp);
	pp^ := t3;

//       *--p = t2;
	Dec(pp);
	pp^ := t2;

//       *--p = t1;
	Dec(pp);
	pp^ := t1;
end; // procedure TIDEA.InvertKeys
{$R+,Q+}

end. // unit IDEA
