//
// $History: MD5.pas $
// 
// *****************  Version 6  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 5  *****************
// User: Lvk          Date: 12.09.98   Time: 17:36
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 4  *****************
// User: Lvk          Date: 9.09.98    Time: 19:15
// Updated in $/Delphi 4.0/StreamFilter
// - Made library Delphi 3.x compatible
// - Fixed bug in encryption base classes
// - Replaced old email address
// 
// *****************  Version 3  *****************
// User: Lvk          Date: 22.07.98   Time: 17:21
// Updated in $/Delphi 4.0/StreamFilter
// Fixed some comment blocks
// 
// *****************  Version 2  *****************
// User: Lvk          Date: 20.07.98   Time: 20:50
// Updated in $/Delphi 4.0/StreamFilter
// Added documentation for first release
// 
// *****************  Version 1  *****************
// User: Lvk          Date: 20.07.98   Time: 18:12
// Created in $/Delphi 4.0/StreamFilter
// Message Digest 5 (MD5) CheckSum class
//

unit MD5;

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net

interface

uses
	AuxFuncs, Math, CheckSum;

type
	TSignature	= array[0..3] of LongWord;
	TBlock			= array[1..64] of Byte;

{ TMD5 class }
	TMD5	= class(TBasicCheckSum)
	private
		FSignature						: TSignature;
		FSignatureCalculated	: Boolean;

		FTempSignature				: TSignature;
		FBlock								: TBlock;
		FInBlock							: Integer;
{$IFDEF VER120}
// Delphi 4.x specific code
		FLength								: Int64;
{$ELSE}
// Delphi 3.x specific code
		FLength								: LongWord;
{$ENDIF}
	protected
		procedure CalculateSignature; virtual;
		procedure FlushBlock; virtual;
		function GetByte(Index: Integer): Byte; override;
	public
		constructor Create;
		procedure Reset; override;
		procedure Process(const Data; Size: Integer); override;
	end; // class TMD5

implementation

resourcestring
	SMD5 = 'Message Digest 5 (MD5)';

{ TMD5 }

const
	S11	=  7;		S12	= 12;		S13	= 17;		S14	= 22;
	S21	=  5;		S22	=  9;		S23	= 14;		S24	= 20;
	S31	=  4;		S32	= 11;		S33	= 16;		S34	= 23;
	S41	=  6;		S42	= 10;		S43	= 15;		S44	= 21;

procedure TMD5.CalculateSignature;
var
	DummyBlock		: array[0..63] of Byte;
	DummyByte			: Byte;
	OldBlock			: TBlock;
	OldInBlock		: Integer;
	OldSignature	: TSignature;
{$IFDEF VER120}
// Delphi 4.x specific code
	OldLength			: Int64;
{$ELSE}
// Delphi 3.x specific code
	Temp					: packed record
		OldLength		: LongWord;
		Zeroes			: LongWord;
	end;
{$ENDIF}
begin
	// No need to recalculate if its already done
	if FSignatureCalculated then
		Exit;

	// Save old data before doing this
	OldBlock := FBlock;
	OldInBlock := FInBlock;
{$IFDEF VER120}
// Delphi 4.x specific code
	OldLength := FLength;
{$ELSE}
// Delphi 3.x specific code
	Temp.OldLength := FLength;
	Temp.Zeroes := 0;
{$ENDIF}
	OldSignature := FTempSignature;

	// We need to pad with 1 1-bit and the rest 0-bits
	DummyByte := 128;
	FillChar(DummyBlock, SizeOf(DummyBlock), 0);

	// Now pad the data
	Process(DummyByte, 1);
	repeat
		if (FInBlock >= 56) then
			Process(DummyBlock, 64-FInBlock)
		else
			Process(DummyBlock, 56-FInBlock);
	until (FInBlock = 56); // repeat

	// And append the length
{$IFDEF VER120}
// Delphi 4.x specific code
	Process(OldLength, SizeOf(OldLength));
{$ELSE}
// Delphi 3.x specific code
	Process(Temp, SizeOf(Temp));
{$ENDIF}

	// Copy the signature
	FSignature := FTempSignature;

	// Restore old data
	FBlock := OldBlock;
	FInBlock := OldInBlock;
{$IFDEF VER120}
// Delphi 4.x specific code
	FLength := OldLength;
{$ELSE}
// Delphi 3.x specific code
	FLength := Temp.OldLength;
{$ENDIF}
	FTempSignature := OldSignature;
	FSignatureCalculated := TRUE;
end; // procedure TMD5.CalculateSignature

constructor TMD5.Create;
begin
	inherited Create(SMD5, SizeOf(FSignature));
end; // constructor TMD5.Create

{$R-,Q-}
procedure TMD5.FlushBlock;
var
	A,B,C,D	: LongWord;
	W				: array[0..15] of LongWord;

	procedure FF(var a: LongWord; b, c, d, x, s, ac: LongWord);

		function F(X, Y, Z: LongWord): LongWord;
		begin
			Result := (X and Y) or ((not X) and Z);
		end; // function TMD5.FlushBlock.FF.F

	begin
		a := a + F(b, c, d) + x + ac;
		a := ROL(a, s);
		a := a + b;
	end; // function TMD5.FlushBlock.FF

	procedure GG(var a: LongWord; b, c, d, x, s, ac: LongWord);

		function G(X, Y, Z: LongWord): LongWord;
		begin
			Result := (X and Z) or (Y and (not Z));
		end; // function TMD5.FlushBlock.GG.G

	begin
		Inc(a, G(b, c, d) + x + ac);
		a := ROL(a, s);
		Inc(a, b);
	end; // function TMD5.FlushBlock.GG

	procedure HH(var a: LongWord; b, c, d, x, s, ac: LongWord);

		function H(X, Y, Z: LongWord): LongWord;
		begin
			Result := X xor Y xor Z;
		end; // function TMD5.FlushBlock.HH.H

	begin
		Inc(a, H(b, c, d) + x + ac);
		a := ROL(a, s);
		Inc(a, b);
	end; // function TMD5.FlushBlock.HH

	procedure II(var a: LongWord; b, c, d, x, s, ac: LongWord);

		function I(X, Y, Z: LongWord): LongWord;
		begin
			Result := (Y xor (X or (not Z)));
		end; // function TMD5.FlushBlock.II.I

	begin
		Inc(a, I(b, c, d) + x + ac);
		a := ROL(a, s);
		Inc(a, b);
	end; // function TMD5.FlushBlock.II

begin
	// Calculate W[]
	Move(FBlock, W, SizeOf(FBlock));

	// Put old signature values in A-D
	A := FTempSignature[0];
	B := FTempSignature[1];
	C := FTempSignature[2];
	D := FTempSignature[3];

	// Do the math
	// Round 1
	FF(A, B, C, D, W[ 0], S11, $d76aa478);
	FF(D, A, B, C, W[ 1], S12, $e8c7b756);
	FF(C, D, A, B, W[ 2], S13, $242070db);
	FF(B, C, D, A, W[ 3], S14, $c1bdceee);
	FF(A, B, C, D, W[ 4], S11, $f57c0faf);
	FF(D, A, B, C, W[ 5], S12, $4787c62a);
	FF(C, D, A, B, W[ 6], S13, $a8304613);
	FF(B, C, D, A, W[ 7], S14, $fd469501);
	FF(A, B, C, D, W[ 8], S11, $698098d8);
	FF(D, A, B, C, W[ 9], S12, $8b44f7af);
	FF(C, D, A, B, W[10], S13, $ffff5bb1);
	FF(B, C, D, A, W[11], S14, $895cd7be);
	FF(A, B, C, D, W[12], S11, $6b901122);
	FF(D, A, B, C, W[13], S12, $fd987193);
	FF(C, D, A, B, W[14], S13, $a679438e);
	FF(B, C, D, A, W[15], S14, $49b40821);

	// Round 2
	GG(A, B, C, D, W[ 1], S21, $f61e2562);
	GG(D, A, B, C, W[ 6], S22, $c040b340);
	GG(C, D, A, B, W[11], S23, $265e5a51);
	GG(B, C, D, A, W[ 0], S24, $e9b6c7aa);
	GG(A, B, C, D, W[ 5], S21, $d62f105d);
	GG(D, A, B, C, W[10], S22, $02441453);
	GG(C, D, A, B, W[15], S23, $d8a1e681);
	GG(B, C, D, A, W[ 4], S24, $e7d3fbc8);
	GG(A, B, C, D, W[ 9], S21, $21e1cde6);
	GG(D, A, B, C, W[14], S22, $c33707d6);
	GG(C, D, A, B, W[ 3], S23, $f4d50d87);
	GG(B, C, D, A, W[ 8], S24, $455a14ed);
	GG(A, B, C, D, W[13], S21, $a9e3e905);
	GG(D, A, B, C, W[ 2], S22, $fcefa3f8);
	GG(C, D, A, B, W[ 7], S23, $676f02d9);
	GG(B, C, D, A, W[12], S24, $8d2a4c8a);

	// Round 3
	HH(A, B, C, D, W[ 5], S31, $fffa3942);
	HH(D, A, B, C, W[ 8], S32, $8771f681);
	HH(C, D, A, B, W[11], S33, $6d9d6122);
	HH(B, C, D, A, W[14], S34, $fde5380c);
	HH(A, B, C, D, W[ 1], S31, $a4beea44);
	HH(D, A, B, C, W[ 4], S32, $4bdecfa9);
	HH(C, D, A, B, W[ 7], S33, $f6bb4b60);
	HH(B, C, D, A, W[10], S34, $bebfbc70);
	HH(A, B, C, D, W[13], S31, $289b7ec6);
	HH(D, A, B, C, W[ 0], S32, $eaa127fa);
	HH(C, D, A, B, W[ 3], S33, $d4ef3085);
	HH(B, C, D, A, W[ 6], S34, $04881d05);
	HH(A, B, C, D, W[ 9], S31, $d9d4d039);
	HH(D, A, B, C, W[12], S32, $e6db99e5);
	HH(C, D, A, B, W[15], S33, $1fa27cf8);
	HH(B, C, D, A, W[ 2], S34, $c4ac5665);

	// Round 4
	II(A, B, C, D, W[ 0], S41, $f4292244);
	II(D, A, B, C, W[ 7], S42, $432aff97);
	II(C, D, A, B, W[14], S43, $ab9423a7);
	II(B, C, D, A, W[ 5], S44, $fc93a039);
	II(A, B, C, D, W[12], S41, $655b59c3);
	II(D, A, B, C, W[ 3], S42, $8f0ccc92);
	II(C, D, A, B, W[10], S43, $ffeff47d);
	II(B, C, D, A, W[ 1], S44, $85845dd1);
	II(A, B, C, D, W[ 8], S41, $6fa87e4f);
	II(D, A, B, C, W[15], S42, $fe2ce6e0);
	II(C, D, A, B, W[ 6], S43, $a3014314);
	II(B, C, D, A, W[13], S44, $4e0811a1);
	II(A, B, C, D, W[ 4], S41, $f7537e82);
	II(D, A, B, C, W[11], S42, $bd3af235);
	II(C, D, A, B, W[ 2], S43, $2ad7d2bb);
	II(B, C, D, A, W[ 9], S44, $eb86d391);

	// Now add it to the signature
	Inc(FTempSignature[0], A);
	Inc(FTempSignature[1], B);
	Inc(FTempSignature[2], C);
	Inc(FTempSignature[3], D);

	// Reset data
	FillChar(FBlock, SizeOf(FBlock), 0);
	FInBlock := 0;
end;
{$R+,Q+}

function TMD5.GetByte(Index: Integer): Byte;
var
	I	: Integer;
begin
	// Check if signature is calculated
	if not FSignatureCalculated then
		CalculateSignature;

	// Find the correct byte
	I := (Index div 4);
	case (Index mod 4) of
		0	: Result := (FSignature[I] and 255);
		1	: Result := (FSignature[I] shr 8) and 255;
		2	: Result := (FSignature[I] shr 16) and 255;
		3	: Result := FSignature[I] shr 24;
	else
		Result := 0;
	end; // case (Index mod 4)
end; // function TMD5.GetByte

procedure TMD5.Process(const Data; Size: Integer);
var
	P				: PChar;
	ToCopy	: Integer;
begin
	P := @Data;
	while (Size > 0) do
	begin
		// Move data to the temporary block
		ToCopy := Min(64 - FInBlock, Size);
		Move(P^, FBlock[FInBlock + 1], ToCopy);
		Inc(FInBlock, ToCopy);
		Inc(P, ToCopy);
		Dec(Size, ToCopy);
		Inc(FLength, ToCopy*8);

		// See if we got one whole block now
		if (FInBlock = 64) then
			FlushBlock;
	end; // while (Size > 0)
end; // procedure TMD5.Process

procedure TMD5.Reset;
const
	InitSignature	: TSignature = (
		$67452301, $efcdab89, $98badcfe, $10325476
	);
begin
	FillChar(FBlock, SizeOf(FBlock), 0);
	FInBlock := 0;
	FSignatureCalculated := FALSE;
	FLength := 0;

	FSignature := InitSignature;
	FTempSignature := InitSignature;
end; // procedure TMD5.Reset

end. // unit MD5
