unit MainForm;

interface

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net
//
// This test program requires a Bitmap called TEST.BMP located in C:\
// You can probably find a suitable bitmap in the Windows directory
//
// Please note that this demo program was made to show off the different
// classes in this package. If this was code that was supposed to be used
// for something, it would have been formatted a lot nicer, so don't
// pester me about it.

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, ComCtrls,

	// Stream filter units
	StreamFilters, zLibStream, CheckSumStream, CryptographyStream, PersistentStorage, 

	// Encryption units
	Cryptography, IDEA, Blowfish, RC5, RC6,

	// Checksum units
	CheckSum, CRC, SHA, MD5;

type
	TfrmStreamsTest = class(TForm)
    pcPages: TPageControl;
    tsEncryption: TTabSheet;
    tsCompression: TTabSheet;
    tsChecksum: TTabSheet;
    lblSourceFilenameCryptography: TLabel;
    editSourceFilenameCryptography: TEdit;
		editDestinationFilenameCryptography: TEdit;
    lblDestinationFilenameCryptography: TLabel;
    lblEncryptionAlgorithm: TLabel;
		cbEncryptionAlgorithm: TComboBox;
		lblEncryptionKey: TLabel;
		editEncryptionKey: TEdit;
		btnEncryptFile: TButton;
		btnDecryptFile: TButton;
		lblSourceFilenameCompression: TLabel;
		lblDestinationFilenameCompression: TLabel;
		editSourceFilenameCompression: TEdit;
		editDestinationFilenameCompression: TEdit;
		btnCompressFile: TButton;
		btnDecompressFile: TButton;
		cbCompressionLevel: TComboBox;
		lblCompressionLevel: TLabel;
		sbStatus: TStatusBar;
    lblFilenameChecksum: TLabel;
    editFilenameChecksum: TEdit;
    cbChecksumAlgorithm: TComboBox;
    lblChecksumAlgorithm: TLabel;
    btnComputeChecksum: TButton;
    lblChecksumResult: TLabel;
    editChecksumResult: TEdit;
		procedure FormCreate(Sender: TObject);
		procedure btnEncryptFileClick(Sender: TObject);
		procedure btnDecryptFileClick(Sender: TObject);
		procedure btnCompressFileClick(Sender: TObject);
		procedure btnDecompressFileClick(Sender: TObject);
    procedure btnComputeChecksumClick(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
	end; // class TfrmStreamsTest

var
	frmStreamsTest: TfrmStreamsTest;

implementation

{$R *.DFM}

resourcestring
	SEncrypting					= 'Encrypting with %s algorithm... please wait';
	SEncrypted					= 'Encrypted';
	SDecrypting					= 'Decrypting with %s algorithm... please wait';
	SDecrypted					= 'Decrypted';

	SCompressing				= 'Compressing... please wait';
	SCompressed					= 'Compressed, compression ratio = %0.2f';
	SDecompressing			= 'Decompressing... please wait';
	SDecompressed				= 'Decompressed, compression ratio = %0.2f';

	SComputingChecksum	= 'Computing checksum... please wait';
	SCheckSumComputed		= 'Checksum computed';

procedure TfrmStreamsTest.FormCreate(Sender: TObject);
begin
	cbEncryptionAlgorithm.ItemIndex := 0;
	cbCompressionLevel.ItemIndex := 0;
	cbChecksumAlgorithm.ItemIndex := 0;
end; // procedure TfrmStreamsTest.FormCreate

procedure TfrmStreamsTest.btnEncryptFileClick(Sender: TObject);
var
	SourceFileStream			: TStream;
	DestinationFileStream	: TStream;
	EncryptionClass				: TCryptography;
	EncryptionStream			: TStream;
begin
	// First open the files
	SourceFileStream := TFileStream.Create(editSourceFilenameCryptography.Text, fmOpenRead);
	DestinationFileStream := TFileStream.Create(editDestinationFilenameCryptography.Text, fmCreate);

	// Now instantiate the encryption class
	case cbEncryptionAlgorithm.ItemIndex of
{$IFDEF VER120}
// Delphi 4.x specific code
		0	: EncryptionClass := TIDEA.Create(editEncryptionKey.Text);
		1	: EncryptionClass := TBlowfish.Create(editEncryptionKey.Text);
		2	: EncryptionClass := TRC5.Create(editEncryptionKey.Text);
		3	: EncryptionClass := TRC6.Create(editEncryptionKey.Text);
{$ELSE}
// Delphi 3.x specific code
		0	: EncryptionClass := TIDEA.CreateKey(editEncryptionKey.Text);
		1	: EncryptionClass := TBlowfish.CreateKey(editEncryptionKey.Text);
		2	: EncryptionClass := TRC5.CreateKey(editEncryptionKey.Text);
		3	: EncryptionClass := TRC6.CreateKey(editEncryptionKey.Text);
{$ENDIF}
	else
		Exit;
	end;

	// Let the user now the program is busy
	sbStatus.SimpleText := Format(SEncrypting, [cbEncryptionAlgorithm.Text]);
	Screen.Cursor := crHourglass;

	// Now create the encryption stream
	EncryptionStream := TEncryptionStream.Create(DestinationFileStream, EncryptionClass, FALSE, FALSE);

	// Encrypt the file
	EncryptionStream.CopyFrom(SourceFileStream, 0);

	// Update the status
	sbStatus.SimpleText := SEncrypted;

	// Free all objects
	EncryptionStream.Free;
	EncryptionClass.Free;
	DestinationFileStream.Free;
	SourceFileStream.Free;

	// Ok, finished now
	Screen.Cursor := crDefault;
end; // procedure TfrmStreamsTest.btnEncryptFileClick

procedure TfrmStreamsTest.btnDecryptFileClick(Sender: TObject);
var
	SourceFileStream			: TStream;
	DestinationFileStream	: TStream;
	EncryptionClass				: TCryptography;
	DecryptionStream			: TStream;
begin
	// First open the files
	SourceFileStream := TFileStream.Create(editSourceFilenameCryptography.Text, fmOpenRead);
	DestinationFileStream := TFileStream.Create(editDestinationFilenameCryptography.Text, fmCreate);

	// Now instantiate the encryption class
	case cbEncryptionAlgorithm.ItemIndex of
{$IFDEF VER120}
// Delphi 4.x specific code
		0	: EncryptionClass := TIDEA.Create(editEncryptionKey.Text);
		1	: EncryptionClass := TBlowfish.Create(editEncryptionKey.Text);
		2	: EncryptionClass := TRC5.Create(editEncryptionKey.Text);
		3	: EncryptionClass := TRC6.Create(editEncryptionKey.Text);
{$ELSE}
// Delphi 3.x specific code
		0	: EncryptionClass := TIDEA.CreateKey(editEncryptionKey.Text);
		1	: EncryptionClass := TBlowfish.CreateKey(editEncryptionKey.Text);
		2	: EncryptionClass := TRC5.CreateKey(editEncryptionKey.Text);
		3	: EncryptionClass := TRC6.CreateKey(editEncryptionKey.Text);
{$ENDIF}
	else
		Exit;
	end;

	// Let the user now the program is busy
	sbStatus.SimpleText := Format(SDecrypting, [cbEncryptionAlgorithm.Text]);
	Screen.Cursor := crHourglass;

	// Now create the encryption stream
	DecryptionStream := TDecryptionStream.Create(SourceFileStream, EncryptionClass, FALSE, FALSE);

	// Encrypt the file
	DestinationFileStream.CopyFrom(DecryptionStream, 0);

	// Update the status
	sbStatus.SimpleText := SDecrypted;

	// Free all objects
	DecryptionStream.Free;
	EncryptionClass.Free;
	DestinationFileStream.Free;
	SourceFileStream.Free;

	// Ok, finished now
	Screen.Cursor := crDefault;
end; // procedure TfrmStreamsTest.btnDecryptFileClick

procedure TfrmStreamsTest.btnCompressFileClick(Sender: TObject);
var
	SourceFileStream			: TStream;
	DestinationFileStream	: TStream;
	CompressionStream			: TzLibCompressionStream;
begin
	// Let the user now the program is busy
	Screen.Cursor := crHourglass;
	sbStatus.SimpleText := SDecompressing;

	// First open the files
	SourceFileStream := TFileStream.Create(editSourceFilenameCompression.Text, fmOpenRead);
	DestinationFileStream := TFileStream.Create(editDestinationFilenameCompression.Text, fmCreate);

	// Create the compression stream
	CompressionStream := TzLibCompressionStream.Create(DestinationFileStream, FALSE,
		TCompressionLevel(cbCompressionLevel.ItemIndex), 32768);

	// Compress the file
	CompressionStream.CopyFrom(SourceFileStream, 0);

	// Update the status
	sbStatus.SimpleText := Format(SCompressed, [CompressionStream.CompressionRatio]);

	// Free all objects
	CompressionStream.Free;
	DestinationFileStream.Free;
	SourceFileStream.Free;

	// Ok, finished now
	Screen.Cursor := crDefault;
end; // procedure TfrmStreamsTest.btnCompressFileClick

procedure TfrmStreamsTest.btnDecompressFileClick(Sender: TObject);
var
	SourceFileStream			: TStream;
	DestinationFileStream	: TStream;
	DecompressionStream		: TzLibDecompressionStream;
begin
	// Let the user now the program is busy
	Screen.Cursor := crHourglass;
	sbStatus.SimpleText := SDecompressing;

	// First open the files
	SourceFileStream := TFileStream.Create(editSourceFilenameCompression.Text, fmOpenRead);
	DestinationFileStream := TFileStream.Create(editDestinationFilenameCompression.Text, fmCreate);

	// Create the compression stream
	DecompressionStream := TzLibDecompressionStream.Create(SourceFileStream, FALSE,
		32768);

	// Compress the file
	DestinationFileStream.CopyFrom(DecompressionStream, 0);

	// Update the status
	sbStatus.SimpleText := Format(SDecompressed, [DecompressionStream.CompressionRatio]);

	// Free all objects
	DecompressionStream.Free;
	DestinationFileStream.Free;
	SourceFileStream.Free;

	// Ok, finished now
	Screen.Cursor := crDefault;
end; // procedure TfrmStreamsTest.btnDecompressFileClick

procedure TfrmStreamsTest.btnComputeChecksumClick(Sender: TObject);
var
	SourceFileStream	: TStream;
	CheckSumClass			: TBasicCheckSum;
	CheckSumStream		: TCheckSumStream;
	Buffer						: array[0..2047] of Byte;
begin
	// First open the file
	SourceFileStream := TFileStream.Create(editFilenameChecksum.Text, fmOpenRead);

	// Create the checksum object
	case cbChecksumAlgorithm.ItemIndex of
		0	: CheckSumClass := TSum8.Create;
		1	: CheckSumClass := TSum16.Create;
		2	: CheckSumClass := TSum32.Create;
		3	: CheckSumClass := TCRC16.Create;
		4	: CheckSumClass := TCRC32.Create;
		5	: CheckSumClass := TMD5.Create;
		6	: CheckSumClass := TSHA.Create;
	else
		Exit;
	end;

	// Create the checksum stream class
	CheckSumStream := TCheckSumStream.Create(SourceFileStream, CheckSumClass, FALSE, FALSE);

	// Let the user now the program is busy
	Screen.Cursor := crHourglass;
	sbStatus.SimpleText := SComputingChecksum;

	// Compute the checksum by reading the whole file
	repeat until (CheckSumStream.Read(Buffer, SizeOf(Buffer)) = 0);

	// Update the status
	sbStatus.SimpleText := SChecksumComputed;
	editCheckSumResult.Text := CheckSumStream.CheckSum;

	// Free all objects
	CheckSumStream.Free;
	CheckSumClass.Free;
	SourceFileStream.Free;

	// Ok, finished now
	Screen.Cursor := crDefault;
end; // procedure TfrmStreamsTest.btnComputeChecksumClick

end. // unit MainForm
