//
// $History: PersistentStorage.pas $
// 
// *****************  Version 4  *****************
// User: Lvk          Date: 21.10.98   Time: 0:37
// Updated in $/Delphi 4.0/StreamFilter
// Changed the persistant storage classes somewhat
// 
// *****************  Version 3  *****************
// User: Lvk          Date: 20.10.98   Time: 23:40
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 2  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 1  *****************
// User: Lvk          Date: 20.10.98   Time: 23:35
// Created in $/Delphi 4.0/StreamFilter
// Persistent Storage classes
// - Implements streams and storages
//

unit PersistentStorage;

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net

interface

uses
	SysUtils, Classes, Windows, ComObj, ActiveX;

type
{ Forward declarations }
	TPersistentStorage		= class;
	TPersistentStream			= class;

{ TPersistentEnumerator class }
	TPersistentEnumerator	= class (TObject, IEnumStatStg)
	private
		FEnumStatStg		: IEnumStatStg;
		FName						: string;
		FValid					: Boolean;
		Fctime					: TFileTime;
		Fatime					: TFileTime;
		Fmtime					: TFileTime;
		FCLSID					: TCLSID;
		FSize						: Int64;
		FType						: LongInt;
		FLocksSupported	: LongInt;
		function IsXXX(Index: Integer): Boolean;
	public
		constructor Create(AStorage: TPersistentStorage); overload;
		destructor Destroy; override;

		property EnumSTATSG: IEnumStatStg read FEnumStatStg implements IEnumStatStg;

		property Valid: Boolean read FValid;
		function Next: Boolean; virtual;

		property Name: string read FName;
		property CreationeTime: TFileTime read Fctime;
		property AccessTime: TFileTime read Fatime;
		property ModificationTime: TFileTime read Fmtime;
		property CLSID: TCLSID read FCLSID;
		property Size: Int64 read FSize;
		property ElementType: LongInt read FType;
		property LocksSupported: LongInt read FLocksSupported;

		property IsStream: Boolean index 0 read IsXXX;
		property IsStorage: Boolean index 1 read IsXXX;
	end; // class TPersistentEnumerator

{ TPersistentStorage class }
	TPersistentStorage		= class (TObject, IStorage)
	private
		FStorage				: IStorage;

		function GetName: string;
		function GetTime(Index: Integer): TFileTime;
		procedure SetTime(const Index: Integer; const Value: TFileTime);

		procedure SetTimeOf(Name: PWideChar; Index: Integer; const Value: TFileTime);
		procedure DoMoveElementTo(AStorage: TPersistentStorage; AName, ANewName: string; DeleteOriginal: Boolean); virtual;
	public
		constructor CreateTemporary(Mode: Integer);
		constructor Create(const AFilename: string; Mode: Integer); overload;
		constructor Create(AStorage: TPersistentStorage; const AName: string; AMode: Integer); overload;

		constructor Open(AFilename: string; Mode: Integer); overload;
		constructor Open(AStorage: TPersistentStorage; const AName: string; AMode: Integer); overload;

		function IsStorage(const Name: string): Boolean;
		function IsStream(const Name: string): Boolean;

		destructor Destroy; override;

		property Storage: IStorage read FStorage implements IStorage;

		procedure CopyTo(AStorage: TPersistentStorage); virtual;
		procedure MoveElementTo(AStorage: TPersistentStorage; const AName, ANewName: string); virtual;
		procedure CopyElementTo(AStorage: TPersistentStorage; const AName, ANewName: string); virtual;

		procedure Commit(grfCommitFlags: Integer); virtual;
		procedure Revert; virtual;

		procedure DestroyElement(const AName: string); virtual;
		procedure Rename(const AName, ANewName: string); virtual;

		procedure SetModificationTimeOfElement(const AName: string; ATime: TFileTime);
		procedure SetCreationTime(const AName: string; ATime: TFileTime);
		procedure SetAccessTime(const AName: string; ATime: TFileTime);

		property Name: string read GetName;
		property ModificationTime: TFileTime index 0 read GetTime write SetTime;
		property CreationTime: TFileTime index 1 read GetTime write SetTime;
		property AccessTime: TFileTime index 2 read GetTime write SetTime;
	end; // class TPersistentStorage

{ TPersistentStream class }
	TPersistentStream			= class(TStream, IStream)
	private
		FStream					: IStream;

		function GetName: string;
		function GetTime(Index: Integer): TFileTime;
		function GetMode: LongInt;
		function GetCLSID: TCLSID;
		function GetLocksSupported: LongInt;
	public
		constructor Create(AStorage: TPersistentStorage; AName: string; AMode: Integer); overload;
		constructor Open(AStorage: TPersistentStorage; AName: string; AMode: Integer); overload;
		constructor Clone(AStream: TPersistentStream);

		destructor Destroy; override;

		property Stream: IStream read FStream implements IStream;

		procedure CopyTo(AStream: TPersistentStream; Length: Int64; BytesRead, BytesWritten: Int64); virtual; 

		function Read(var Buffer; Count: Longint): Longint; override;
		function Write(const Buffer; Count: Longint): Longint; override;
		function Seek(Offset: Longint; Origin: Word): Longint; override;
		procedure SetSize(NewSize: Integer); override;

		function Seek64(Offset: Int64; Origin: Word): Int64; virtual;
		procedure SetSize64(NewSize: Integer); virtual;

		procedure LockRegion(Offset, Length: Int64; LockType: Integer); virtual;
		procedure UnlockRegion(Offset, Length: Int64; LockType: Integer); virtual;

		procedure Commit(grfCommitFlags: Integer); virtual;
		procedure Revert; virtual;

		property Name: string read GetName;
		property ModificationTime: TFileTime index 0 read GetTime;
		property CreationTime: TFileTime index 1 read GetTime;
		property AccessTime: TFileTime index 2 read GetTime;
		property Mode: LongInt read GetMode;
		property CLSID: TCLSID read GetCLSID;
		property LocksSupported: LongInt read GetLocksSupported;
	end; // class TPersistentStream

procedure OleCheckFmt(OleResult: HResult; const Params: array of const);

implementation

procedure OleCheckFmt(OleResult: HResult; const Params: array of const);
var
	S		: string;
	S2	: string;
	x		: Integer;
	Num	: Integer;
begin
	try
		OleCheck(OleResult);
	except
		on E: Exception do
		begin
			S := E.Message;
			while (Pos('%', S) > 0) do
			begin
				x := Pos('%', S);
				Delete(S, x, 1);
				Num := 0;
				while ((x <= Length(S)) and (S[x] in ['0'..'9'])) do
				begin
					Num := (Num * 10) + (Ord(S[x]) - Ord('0'));
					Delete(S, x, 1);
				end; // while
				S2 := Params[x - 1].VPChar;
				Insert(S2, S, x);

			end; // while
			raise EOleError.Create(S);
		end; // on Exception
	end; // try
end; // procedure OleCheckFmt

{ TPersistentStorage }

constructor TPersistentStorage.Create(const AFilename: string; Mode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AFilename, Temp, SizeOf(Temp));
	OleCheckFmt(StgCreateDocFile(Temp, Mode, 0, FStorage), [AFilename]);
	inherited Create;
end; // constructor TPersistentStorage.Create(string, Integer)

procedure TPersistentStorage.CopyTo(AStorage: TPersistentStorage);
begin
	OleCheck(FStorage.CopyTo(0, nil, nil, AStorage.Storage));
end; // procedure TPersistentStorage.CopyTo

constructor TPersistentStorage.Create(AStorage: TPersistentStorage; const AName: string; AMode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	OleCheckFmt(AStorage.Storage.CreateStorage(Temp, AMode, 0, 0, FStorage), [AName]);
	inherited Create;
end; // constructor TPersistentStorage.Create(TPersistentStorage, string, Integer)

destructor TPersistentStorage.Destroy;
begin
	inherited Destroy;
	FStorage := nil;
end; // destructor TPersistentStorage.Destroy

constructor TPersistentStorage.Open(AFilename: string; Mode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AFilename, Temp, SizeOf(Temp));
	OleCheckFmt(StgOpenStorage(Temp, nil, Mode, nil, 0, FStorage), [AFilename]);
	inherited Create;
end; // constructor TPersistentStorage.Open(string, Integer)

constructor TPersistentStorage.Open(AStorage: TPersistentStorage; const AName: string; AMode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	OleCheckFmt(AStorage.Storage.OpenStorage(Temp, nil, AMode, nil, 0, FStorage), [AName]);
	inherited Create;
end; // constructor TPersistentStorage.Open(TPersistentStorage, string, Integer)

procedure TPersistentStorage.Commit(grfCommitFlags: Integer);
begin
	OleCheck(FStorage.Commit(grfCommitFlags));
end; // procedure TPersistentStorage.Commit

procedure TPersistentStorage.Revert;
begin
	OleCheck(FStorage.Revert);
end; // procedure TPersistentStorage.Revert

procedure TPersistentStorage.DestroyElement(const AName: string);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	OleCheckFmt(FStorage.DestroyElement(Temp), [AName]);
end; // procedure TPersistentStorage.DestroyElement

procedure TPersistentStorage.Rename(const AName, ANewName: string);
var
	TempName		: array[0..1023] of WideChar;
	TempNewName	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, TempName, SizeOf(TempName));
	StringToWideChar(ANewName, TempNewName, SizeOf(TempNewName));
	OleCheckFmt(FStorage.RenameElement(TempName, TempNewName), [AName, ANewName]);
end; // procedure TPersistentStorage.Rename

function TPersistentStorage.GetName: string;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStorage.Stat(StatStruc, STATFLAG_DEFAULT));
	Result := StatStruc.pwcsName;
	CoTaskMemFree(StatStruc.pwcsName);
end; // function TPersistentStorage.GetName

function TPersistentStorage.GetTime(Index: Integer): TFileTime;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStorage.Stat(StatStruc, STATFLAG_NONAME));
	case Index of
		0	: Result := StatStruc.mtime;
		1	: Result := StatStruc.ctime;
		2	: Result := StatStruc.atime;
	end; // case
end; // function TPersistentStorage.GetTime

procedure TPersistentStorage.SetTime(const Index: Integer; const Value: TFileTime);
begin
	SetTimeOf(nil, Index, Value);
end; // procedure TPersistentStorage.SetTime

procedure TPersistentStorage.SetAccessTime(const AName: string; ATime: TFileTime);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	SetTimeOf(Temp, 2, ATime);
end; // procedure TPersistentStorage.SetAccessTime

procedure TPersistentStorage.SetCreationTime(const AName: string; ATime: TFileTime);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	SetTimeOf(Temp, 1, ATime);
end; // procedure TPersistentStorage.SetCreationTime

procedure TPersistentStorage.SetModificationTimeOfElement(const AName: string; ATime: TFileTime);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	SetTimeOf(Temp, 0, ATime);
end; // procedure TPersistentStorage.SetModificationTimeOfElement

procedure TPersistentStorage.SetTimeOf(Name: PWideChar; Index: Integer; const Value: TFileTime);
var
	ctime	: ^TFileTime;
	atime	: ^TFileTime;
	mtime	: ^TFileTime;
begin
	ctime := nil;
	atime := nil;
	mtime := nil;
	case Index of
		0	: mtime := @Value;
		1	: ctime := @Value;
		2	: atime := @Value;
	end; // case
	OleCheck(FStorage.SetElementTimes(Name, ctime^, atime^, mtime^));
end; // procedure TPersistentStorage.SetTimeOf

procedure TPersistentStorage.CopyElementTo(AStorage: TPersistentStorage; const AName, ANewName: string);
begin
	DoMoveElementTo(AStorage, AName, ANewName, False);
end; // procedure TPersistentStorage.CopyElementTo

procedure TPersistentStorage.DoMoveElementTo(AStorage: TPersistentStorage; AName, ANewName: string; DeleteOriginal: Boolean);
var
	Mode				: Integer;
	TempName		: array[0..1023] of WideChar;
	TempNewName	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, TempName, SizeOf(TempName));
	StringToWideChar(ANewName, TempNewName, SizeOf(TempNewName));

	// Select correct mode
	Mode := STGMOVE_COPY;
	if DeleteOriginal then
		Mode := STGMOVE_MOVE;
		
	OleCheckFmt(FStorage.MoveElementTo(TempName, AStorage.Storage, TempNewName, Mode), [AName, ANewName]);
end; // procedure TPersistentStorage.DoMoveElementTo

procedure TPersistentStorage.MoveElementTo(AStorage: TPersistentStorage; const AName, ANewName: string);
begin
	DoMoveElementTo(AStorage, AName, ANewName, True);
end; // procedure TPersistentStorage.MoveElementTo

constructor TPersistentStorage.CreateTemporary(Mode: Integer);
begin
	OleCheck(StgCreateDocFile(nil, Mode or STGM_DELETEONRELEASE, 0, FStorage));
	inherited Create;
end; // constructor TPersistentStorage.CreateTemporary

function TPersistentStorage.IsStorage(const Name: string): Boolean;
var
	Storage	: TPersistentStorage;
begin
	Result := True;
	Storage := nil;
	try
		try
			Storage := TPersistentStorage.Open(Self, Name, STGM_READ);
		except
			on Exception do
			begin
				Result := False;
			end; // on Exception
		end; // try except
	finally
		Storage.Free;
	end; // try finally
end; // function TPersistentStorage.IsStorage

function TPersistentStorage.IsStream(const Name: string): Boolean;
var
	Stream	: TPersistentStream;
begin
	Result := True;
	Stream := nil;
	try
		try
			Stream := TPersistentStream.Open(Self, Name, STGM_READ);
		except
			on Exception do
			begin
				Result := False;
			end; // on Exception
		end; // try except
	finally
		Stream.Free;
	end; // try finally
end;

{ TPersistentStream }

constructor TPersistentStream.Clone(AStream: TPersistentStream);
begin
	OleCheck(AStream.Stream.Clone(FStream));
	inherited Create;
end; // constructor TPersistentStream.Clone

procedure TPersistentStream.Commit(grfCommitFlags: Integer);
begin
	OleCheck(FStream.Commit(grfCommitFlags));
end; // procedure TPersistentStream.Commit

procedure TPersistentStream.CopyTo(AStream: TPersistentStream; Length: Int64; BytesRead, BytesWritten: Int64);
begin
	OleCheck(FStream.CopyTo(AStream.Stream, Length, BytesRead, BytesWritten));
end; // procedure TPersistentStream.CopyTo

constructor TPersistentStream.Create(AStorage: TPersistentStorage; AName: string; AMode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	OleCheckFmt(AStorage.Storage.CreateStream(Temp, AMode, 0, 0, FStream), [AName]);
	inherited Create;
end; // constructor TPersistentStream.Create

destructor TPersistentStream.Destroy;
begin
	inherited Destroy;
	FStream := nil;
end; // destructor TPersistentStream.Destroy

function TPersistentStream.GetCLSID: TCLSID;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStream.Stat(StatStruc, STATFLAG_NONAME));
	Result := StatStruc.CLSID;
end; // function TPersistentStream.GetCLSID

function TPersistentStream.GetName: string;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStream.Stat(StatStruc, STATFLAG_DEFAULT));
	Result := StatStruc.pwcsName;
	CoTaskMemFree(StatStruc.pwcsName);
end; // function TPersistentStream.GetFilename

function TPersistentStream.GetLocksSupported: LongInt;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStream.Stat(StatStruc, STATFLAG_NONAME));
	Result := StatStruc.grfLocksSupported;
end; // function TPersistentStream.GetLocksSupported

function TPersistentStream.GetMode: LongInt;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStream.Stat(StatStruc, STATFLAG_NONAME));
	Result := StatStruc.grfMode;
end; // function TPersistentStream.GetMode

function TPersistentStream.GetTime(Index: Integer): TFileTime;
var
	StatStruc	: TStatStg;
begin
	OleCheck(FStream.Stat(StatStruc, STATFLAG_NONAME));
	case Index of
		0	: Result := StatStruc.mtime;
		1	: Result := StatStruc.ctime;
		2	: Result := StatStruc.atime;
	end; // case
end; // function TPersistentStream.GetTime

procedure TPersistentStream.LockRegion(Offset, Length: Int64; LockType: Integer);
begin
	OleCheck(FStream.LockRegion(Offset, Length, LockType));
end; // procedure TPersistentStream.LockRegion

constructor TPersistentStream.Open(AStorage: TPersistentStorage; AName: string; AMode: Integer);
var
	Temp	: array[0..1023] of WideChar;
begin
	StringToWideChar(AName, Temp, SizeOf(Temp));
	OleCheckFmt(AStorage.Storage.OpenStream(Temp, nil, AMode, 0, FStream), [AName]);
	inherited Create;
end; // constructor TPersistentStream.Open

function TPersistentStream.Read(var Buffer; Count: Integer): Longint;
begin
	OleCheck(FStream.Read(@Buffer, Count, @Result));
end; // function TPersistentStream.Read

procedure TPersistentStream.Revert;
begin
	OleCheck(FStream.Revert);
end; // procedure TPersistentStream.Revert

function TPersistentStream.Seek(Offset: Integer; Origin: Word): Longint;
begin
	Result := Seek64(Offset, Origin);
end; // function TPersistentStream.Seek

function TPersistentStream.Seek64(Offset: Int64; Origin: Word): Int64;
begin
	OleCheck(FStream.Seek(Offset, Origin, Result));
end; // function TPersistentStream.Seek64

procedure TPersistentStream.SetSize(NewSize: Integer);
begin
	SetSize64(NewSize);
end; // procedure TPersistentStream.SetSize

procedure TPersistentStream.SetSize64(NewSize: Integer);
begin
	OleCheck(FStream.SetSize(NewSize));
end; // procedure TPersistentStream.SetSize64

procedure TPersistentStream.UnlockRegion(Offset, Length: Int64; LockType: Integer);
begin
	OleCheck(FStream.UnlockRegion(Offset, Length, LockType));
end; // procedure TPersistentStream.UnlockRegion

function TPersistentStream.Write(const Buffer; Count: Integer): Longint;
begin
	OleCheck(FStream.Write(@Buffer, Count, @Result));
end; // function TPersistentStream.Write

{ TPersistentEnumerator }

constructor TPersistentEnumerator.Create(AStorage: TPersistentStorage);
begin
	OleCheck(AStorage.FStorage.EnumElements(0, nil, 0, FEnumStatStg));
	inherited Create;

	Next;
end; // constructor TPersistentEnumerator.Create

destructor TPersistentEnumerator.Destroy;
begin
	FEnumStatStg := nil;
end; // destructor TPersistentEnumerator.Destroy

function TPersistentEnumerator.IsXXX(Index: Integer): Boolean;
begin
	case Index of
		0	: Result := ((ElementType and STGTY_STREAM) <> 0);
		1	: Result := ((ElementType and STGTY_STORAGE) <> 0);
	else
		Result := False;
	end; // case
end; // function TPersistentEnumerator.IsXXX

function TPersistentEnumerator.Next: Boolean;
var
	StatStg	: TStatStg;
begin
	FillChar(StatStg, SizeOf(StatStg), #0);
	FValid := (FEnumStatStg.Next(1, StatStg, nil) = S_OK);
	Result := FValid;

	if Valid then
	begin
		if Assigned(StatStg.pwcsName) then
		begin
			FName := WideCharToString(StatStg.pwcsName);
			CoTaskMemFree(StatStg.pwcsName);
		end // if
		else
			FName := '';

		Fctime := StatStg.ctime;
		Fatime := StatStg.atime;
		Fmtime := StatStg.mtime;

		FCLSID := StatStg.CLSID;
		FSize := StatStg.cbSize;
		FType := StatStg.dwType;
		FLocksSupported := StatStg.grfLocksSupported;
	end; // if
end; // function TPersistentEnumerator.Next

end. // unit PersistentStorage

