//
// $History: zLib.pas $
// 
// *****************  Version 12  *****************
// User: Lvk          Date: 20.10.98   Time: 23:37
// Updated in $/Delphi 4.0/StreamFilter
// Modified email address + added persistent storage classes
// 
// *****************  Version 11  *****************
// User: Lvk          Date: 12.09.98   Time: 17:40
// Updated in $/Delphi 4.0/Indexer
// 
// *****************  Version 10  *****************
// User: Lvk          Date: 12.09.98   Time: 17:36
// Updated in $/Delphi 4.0/StreamFilter
// 
// *****************  Version 9  *****************
// User: Lvk          Date: 9.09.98    Time: 19:15
// Updated in $/Delphi 4.0/StreamFilter
// - Made library Delphi 3.x compatible
// - Fixed bug in encryption base classes
// - Replaced old email address
// 
// *****************  Version 8  *****************
// User: Administrator Date: 29.08.98   Time: 13:31
// Updated in $/Delphi 4.0/Indexer
// Changed email address
// 
// *****************  Version 7  *****************
// User: Lvk          Date: 20.07.98   Time: 20:50
// Updated in $/Delphi 4.0/StreamFilter
// Added documentation for first release
// 
// *****************  Version 6  *****************
// User: Lvk          Date: 17.07.98   Time: 23:54
// Updated in $/Delphi 4.0/StreamFilter
// Added copyright message
// 
// *****************  Version 5  *****************
// User: Lvk          Date: 17.07.98   Time: 20:57
// Updated in $/Delphi 4.0/zLib
// Added history list
//

unit zLib;

{ zlib.h -- interface of the 'zlib' general purpose compression library
	version 1.1.3, July 9th, 1998

	Copyright (C) 1995-1998 Jean-loup Gailly and Mark Adler

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		 claim that you wrote the original software. If you use this software
		 in a product, an acknowledgment in the product documentation would be
		 appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		 misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.

	Jean-loup Gailly        Mark Adler
	jloup@gzip.org          madler@alumni.caltech.edu


	The data format used by the zlib library is described by RFCs (Request for
	Comments) 1950 to 1952 in the files ftp://ds.internic.net/rfc/rfc1950.txt
	(zlib format), rfc1951.txt (deflate format) and rfc1952.txt (gzip format).

	zLib.pas -- Delphi 2.0-4.0 interface for the pre-built 'zlib' DLL available
	at http://www.winimage.com/zLibDll/.

	This code is free under the same terms and conditions as 'zlib.'

	Delphi version originally created by Don Hatlestad:

	Copyright (c) 1996 Don Hatlestad, Art Anderson Associates

	Version updated by Lasse Vgsther Karlsen to conform to changes done to
	the c release of zLib, up to and including version 1.1.3. Also, all
	comments and unnecessary code, constants and structures have been
	removed from this source file, and it has been generally tidied up.

	If you wish to read the comments for each function, look them up in the
	zLib.h file which follows this zLib.pas file. zLib.h is the 1.1.3 header
	file from the original zlib package.

	zLib is available from these sources:
		Official Homepage:			http://www.cdrom.com/pub/infozip/zlib/zlib.html
		zLib.DLL homepage:			http://www.winimage.com/zLibDll/
		zLib.pas homepage:			http://home.sol.no/~lassevk/zlib.htm
}

// (C) Lasse Vgsther Karlsen, 1998
// lasse.karlsen@c2i.net

interface

type
{$IFDEF VER100}
	LongWord				= Cardinal;
{$ENDIF}
	PInteger				= ^Integer;
	uInt						= LongWord;
	uLong						= LongWord;
	uLongf					= ULONG;
	puLongf					= ^ULONG;
	voidpf					= Pointer;
	voidp						= Pointer;
	z_off_t					= LongInt;
	crc_table				= array[Byte] of uLongf;
	pcrc_table			= ^crc_table;
	TGZFILE					= VOIDP;
	alloc_func			= function(opaque : voidpf; items : uInt; size : uInt) : Pointer; stdcall;
	free_func				= procedure(opaque : voidpf; address : voidpf); stdcall;
	internal_state	= record end;
	pinternal_state	= ^internal_state;
	z_stream				= record
		next_in				: Pointer;
		avail_in			: uInt;
		total_in			: uLong;
		next_out			: Pointer;
		avail_out			: uInt;
		total_out			: uLong;
		msg						: PChar;
		state					: pinternal_state;
		zalloc				: alloc_func;
		zfree					: free_func;
		opaque				: Pointer;
		data_type			: Integer;
		adler					: uLong;
		reserved			: uLong;
	end;

const
	SEEK_SET							= 0;
	SEEK_CUR							= 1;
	ZLIB_VERSION					: PChar = '1.1.3';

	Z_NO_FLUSH						= 0;
	Z_PARTIAL_FLUSH				= 1;
	Z_SYNC_FLUSH					= 2;
	Z_FULL_FLUSH					= 3;
	Z_FINISH							= 4;

	Z_OK									= 0;
	Z_STREAM_END					= 1;
	Z_NEED_DICT						= 2;
	Z_ERRNO								= (-1);
	Z_STREAM_ERROR				= (-2);
	Z_DATA_ERROR					= (-3);
	Z_MEM_ERROR						= (-4);
	Z_BUF_ERROR						= (-5);
	Z_VERSION_ERROR				= (-6);

	Z_NO_COMPRESSION			= 0;
	Z_BEST_SPEED					= 1;
	Z_BEST_COMPRESSION		= 9;
	Z_DEFAULT_COMPRESSION	= (-1);

	Z_FILTERED						= 1;
	Z_HUFFMAN_ONLY				= 2;
	Z_DEFAULT_STRATEGY		= 0;

	Z_BINARY							= 0;
	Z_ASCII								= 1;
	Z_UNKNOWN							= 2;

	Z_DEFLATED						= 8;

	Z_NULL								= nil;

	ZLIBDLL								= 'zLib.dll'; 

{ ================================================================= }
function zlibVersion
				: PChar;
					stdcall;

{ ================================================================= }
function deflateInit(							var	strm		: z_stream;
																			level		: Integer)
				: Integer;

{ ================================================================= }
function deflate(									var strm		: z_stream;
																			flush		: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateEnd(							var strm		: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateInit(							var strm		: z_stream)
				: Integer;

{ ================================================================= }
function inflate(									var strm		: z_stream;
																			flush		: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateEnd(							var strm		: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateInit2(						var strm				: z_stream;
																			level				: Integer;
																			method			: Integer;
																			windowBits	: Integer;
																			memLevel		: Integer;
																			strategy		: Integer)
				: Integer;

{ ================================================================= }
function deflateSetDictionary(		var strm				: z_stream;
																			dictionary	: PChar;
																			dictLength	: uInt)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateCopy(							var dest				: z_stream;
																const source			: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateReset(						var strm				: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateParams(						var strm				: z_stream;
																			level				: Integer;
																			strategy		: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function InflateInit2(						var strm				: z_stream;
																			windowBits	: Integer)
				: Integer;

{ ================================================================= }
function inflateSetDictionary(		var strm				: z_stream;
																			dictionary	: PChar;
																			dictLength	: uInt)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateSync(							var strm				: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateReset(						var strm				: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function compress(										dest				: Pointer;
																	var destLen			: uLong;
																			source			: Pointer;
																			sourceLen		: ULONG)
				: Integer;
					stdcall;

{ ================================================================= }
function compress2(										dest				: Pointer;
																	var destLen			: uLong;
																			source			: Pointer;
																			sourceLen		: ULONG;
																			level				: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function uncompress(									dest				: Pointer;
																	var destLen			: uLong;
																			source			: Pointer;
																			sourceLen		: LongInt)
				: Integer;
					stdcall;

{ ================================================================= }
function gzopen(											path				: PChar;
																			mode				: PChar)
				: TGZFILE;
					stdcall;

{ ================================================================= }
function gzdopen(											fd					: Integer;
																			mode				: PChar)
				: TGZFILE;
					stdcall;

{ ================================================================= }
function gzetparams(									gzFile			: TGZFILE;
																			level				: Integer;
																			strategy		: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function gzread(											gzFile			: TGZFILE;
																			buf					: VOIDP;
																			len					: uInt)
				: SmallInt;
					stdcall;

{ ================================================================= }
function gzwrite(											gzFile			: TGZFILE;
																			buf					: VOIDP;
																			len					: uInt)
				: Integer;
					stdcall;

{ ================================================================= }
function gzputs(											gzFile			: TGZFILE;
																			s						: PChar)
				: Integer;
					stdcall;

{ ================================================================= }
function gzgets(											gzFile			: TGZFILE;
																			buf					: PChar;
																			len					: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function gzputc(											gzFile			: TGZFILE;
																			c						: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function gzgetc(											gzFile			: TGZFILE)
				: Integer;
					stdcall;

{ ================================================================= }
function gzflush(											gzFile			: TGZFILE;
																			flush				: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function gzseek(											gzFile			: TGZFILE;
																			offset			: z_off_t;
																			whence			: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function gzrewind(										gzFile			: TGZFILE)
				: Integer;
					stdcall;

{ ================================================================= }
function gztell(											gzFile			: TGZFILE)
				: z_off_t;
					stdcall;

{ ================================================================= }
function gzeof(												gzFile			: TGZFILE)
				: WordBool;
					stdcall;

{ ================================================================= }
function gzclose(											gzFile			: TGZFILE)
				: Integer;
					stdcall;

{ ================================================================= }
function gzerror(											gzFile			: TGZFILE;
																			errnum			: PInteger)
				: PChar;
					stdcall;

{ ================================================================= }
function adler32(											adler				: ULONG;
																			buf					: Pointer;
																			len					: uInt)
				: ULONG;
					stdcall;

{ ================================================================= }
function crc32(												crc					: ULONG;
																			buf					: Pointer;
																			len					: uInt)
				: ULONG;
					stdcall;

{ ================================================================= }
function deflateInit_(						var strm				: z_stream;
																			level				: Integer;
																			version			: PChar;
																			stream_size	: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateInit_(						var strm				: z_stream;
																			version			: PChar;
																			stream_size	: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function deflateInit2_(						var strm				: z_stream;
																			level				: Integer;
																			method			: Integer;
																			windowBits	: Integer;
																			memLevel		: Integer;
																			strategy		: Integer;
																			version			: PChar;
																			stream_size	: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function inflateInit2_(						var strm				: z_stream;
																			windowBits	: Integer;
																			version			: PChar;
																			stream_size	: Integer)
				: Integer;
					stdcall;

{ ================================================================= }
function zError(											err					: Integer)
				: PChar;
					stdcall;

{ ================================================================= }
function inflateSyncPoint(				var z						: z_stream)
				: Integer;
					stdcall;

{ ================================================================= }
function get_crc_table
				: pcrc_table;
					stdcall	;

{ ================================================================= }
{ Functions added by Lasse Vgsther Karlsen for easier programming }

function z_check_version
				: Integer;
	{ Performs version check on the dll }
const
	zVersionOK		= 0;	// DLL version is the same version as this unit supports
	zVersionOlder	= 1;	// The DLL is an older version than
	zVersionNewer	= 2;	// The DLL is newer than what this unit supports. The program should probably not attempt
											// to use the zLib.dll functions

procedure z_init_zstream(					var strm				: z_stream);
	{ Clears the z_stream variable }

implementation

uses
	SysUtils;

{ Support functions by LVK }

function z_check_version: Integer;
begin
	Result:=StrComp(ZLIB_VERSION,zLibVersion);
	if Result<0 then
		Result:=zVersionNewer
	else if Result>0 then
		Result:=zVersionOlder
	else
		Result:=zVersionOK;
end;

procedure z_init_zstream(var strm: z_stream);
begin
	with strm do begin
		next_in:=nil;
		avail_in:=0;
		total_in:=0;
		next_out:=nil;
		avail_out:=0;
		total_out:=0;
		msg:=nil;
		state:=nil;
		zalloc:=nil;
		zfree:=nil;
		opaque:=nil;
		data_type:=0;
		adler:=0;
		reserved:=0;
	end;
end;

{ Import functions from zLib.dll }

function zlibVersion;										external ZLIBDLL;
function deflate;												external ZLIBDLL;
function deflateEnd;										external ZLIBDLL;
function inflate;												external ZLIBDLL;
function inflateEnd;										external ZLIBDLL;
function deflateSetDictionary;					external ZLIBDLL;
function deflateCopy;										external ZLIBDLL;
function deflateReset;									external ZLIBDLL;
function deflateParams;									external ZLIBDLL;
function inflateSetDictionary;					external ZLIBDLL;
function inflateSync;										external ZLIBDLL;
function inflateReset;									external ZLIBDLL;
function compress;											external ZLIBDLL;
function compress2;											external ZLIBDLL;
function uncompress;										external ZLIBDLL;
function gzopen;												external ZLIBDLL;
function gzdopen;												external ZLIBDLL;
function gzetparams;										external ZLIBDLL;
function gzread;												external ZLIBDLL;
function gzwrite;												external ZLIBDLL;
function gzputs;												external ZLIBDLL;
function gzgets;												external ZLIBDLL;
function gzputc;												external ZLIBDLL;
function gzgetc;												external ZLIBDLL;
function gzflush;												external ZLIBDLL;
function gzseek;												external ZLIBDLL;
function gzrewind;											external ZLIBDLL;
function gztell;												external ZLIBDLL;
function gzeof;													external ZLIBDLL;
function gzclose;												external ZLIBDLL;
function gzerror;												external ZLIBDLL;
function adler32;												external ZLIBDLL;
function crc32;													external ZLIBDLL;
function deflateInit_;									external ZLIBDLL;
function inflateInit_;									external ZLIBDLL;
function deflateInit2_;									external ZLIBDLL;
function inflateInit2_;									external ZLIBDLL;
function zError;												external ZLIBDLL;
function inflateSyncPoint;							external ZLIBDLL;
function get_crc_table;									external ZLIBDLL;

{ Support functions }

function deflateInit(var strm: z_stream; level : Integer) : Integer;
begin
	result:=deflateInit_(strm, level, ZLIB_VERSION, SizeOf(z_stream));
end;

function inflateInit(var strm: z_stream) : Integer;
begin
	result:=inflateInit_(strm, ZLIB_VERSION, SizeOf(z_stream));
end;

function deflateInit2(var strm: z_stream; level: Integer; method: Integer; windowBits: Integer;
	memLevel: Integer; strategy: Integer) : Integer;
begin
	result:=deflateInit2_(strm, level, method, windowBits, memLevel, strategy, ZLIB_VERSION, SizeOf(z_stream));
end;

function inflateInit2(var strm: z_stream; windowBits: Integer) : Integer;
begin
	result:=inflateInit2_(strm, windowBits, ZLIB_VERSION, SizeOf(z_stream));
end;

end.


